<?php
/***********************************************
* File      :   config.php
* Project   :   Z-Push
* Descr     :   LDAP backend configuration file
*
* Created   :   27.11.2012
*
* Copyright 2007 - 2016 Zarafa Deutschland GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License, version 3,
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* Consult LICENSE file for details
************************************************/

// **********************
//  BackendLDAP settings
// **********************

// Server address
define('LDAP_SERVER', 'localhost');

// Server Port
define('LDAP_SERVER_PORT', '389');

// LDAP USER DN
define('LDAP_USER_DN', 'uid=%u,ou=mailaccount,dc=phppush,dc=com');

// LDAP BASE DNS
define('LDAP_BASE_DNS', 'Contacts:ou=addressbook,uid=%u,ou=mailaccount,dc=phppush,dc=com'); //Multiple values separator is |
