<?php

// autoload_files.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    '158e247719544c05f5e89c414f630c24' => $baseDir . '/version.php',
    '7e65a9fc8bb44d8c2fe16fa283aeaaee' => $baseDir . '/lib/core/zpushdefs.php',
    'd2a63a53b4a43a2bd71de0cec5c1abfb' => $baseDir . '/lib/utils/compat.php',
);
